@@ -1,11 +1,12 @@
 /* -*- c-basic-offset: 8 -*-
    rdesktop: A Remote Desktop Protocol client.
    Disk Redirection
-   Copyright (C) Jeroen Meijer 2003
+   Copyright (C) Jeroen Meijer <jeroen@oldambt7.com> 2003-2008
+   Copyright 2003-2011 Peter Astrand <astrand@cendio.se> for Cendio AB
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -14,8 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "disk.h"
@@ -121,7 +121,7 @@ struct dummy_statfs_t
 	int f_namemax;
 };
 
-int
+static int
 dummy_statfs(struct dummy_statfs_t *buf)
 {
 	buf->f_blocks = 262144;
@@ -140,25 +140,25 @@ dummy_statfs(struct dummy_statfs_t *buf)
 extern RDPDR_DEVICE g_rdpdr_device[];
 
 FILEINFO g_fileinfo[MAX_OPEN_FILES];
-BOOL g_notify_stamp = False;
+RD_BOOL g_notify_stamp = False;
 
 typedef struct
 {
-	char name[256];
-	char label[256];
+	char name[PATH_MAX];
+	char label[PATH_MAX];
 	unsigned long serial;
-	char type[256];
+	char type[PATH_MAX];
 } FsInfoType;
 
-static NTSTATUS NotifyInfo(NTHANDLE handle, uint32 info_class, NOTIFY * p);
+static RD_NTSTATUS NotifyInfo(RD_NTHANDLE handle, uint32 info_class, NOTIFY * p);
 
 static time_t
-get_create_time(struct stat *st)
+get_create_time(struct stat *filestat)
 {
 	time_t ret, ret1;
 
-	ret = MIN(st->st_ctime, st->st_mtime);
-	ret1 = MIN(ret, st->st_atime);
+	ret = MIN(filestat->st_ctime, filestat->st_mtime);
+	ret1 = MIN(ret, filestat->st_atime);
 
 	if (ret1 != (time_t) 0)
 		return ret1;
@@ -201,7 +201,7 @@ ftruncate_growable(int fd, off_t length)
 {
 	int ret;
 	off_t pos;
-	static const char zero;
+	static const char zero = 0;
 
 	/* Try the simple method first */
 	if ((ret = ftruncate(fd, length)) != -1)
@@ -209,7 +209,7 @@ ftruncate_growable(int fd, off_t length)
 		return ret;
 	}
 
-	/* 
+	/*
 	 * Some kind of error. Perhaps we were trying to grow. Retry
 	 * in a safe way.
 	 */
@@ -262,7 +262,7 @@ static int
 open_weak_exclusive(const char *pathname, int flags, mode_t mode)
 {
 	int ret;
-	struct stat statbuf;
+	struct stat filestat;
 
 	ret = open(pathname, flags, mode);
 	if (ret != -1 || !(flags & O_EXCL))
@@ -291,7 +291,7 @@ open_weak_exclusive(const char *pathname, int flags, mode_t mode)
 	}
 
 	/* Retry with GUARDED semantics */
-	if (stat(pathname, &statbuf) != -1)
+	if (stat(pathname, &filestat) != -1)
 	{
 		/* File exists */
 		errno = EEXIST;
@@ -325,7 +325,7 @@ disk_enum_devices(uint32 * id, char *optarg)
 			fprintf(stderr, "share name %s truncated to %s\n", optarg,
 				g_rdpdr_device[*id].name);
 
-		g_rdpdr_device[*id].local_path = xmalloc(strlen(pos2) + 1);
+		g_rdpdr_device[*id].local_path = (char *) xmalloc(strlen(pos2) + 1);
 		strcpy(g_rdpdr_device[*id].local_path, pos2);
 		g_rdpdr_device[*id].device_type = DEVICE_TYPE_DISK;
 		count++;
@@ -337,14 +337,14 @@ disk_enum_devices(uint32 * id, char *optarg)
 }
 
 /* Opens or creates a file or directory */
-static NTSTATUS
+static RD_NTSTATUS
 disk_create(uint32 device_id, uint32 accessmask, uint32 sharemode, uint32 create_disposition,
-	    uint32 flags_and_attributes, char *filename, NTHANDLE * phandle)
+	    uint32 flags_and_attributes, char *filename, RD_NTHANDLE * phandle)
 {
-	NTHANDLE handle;
+	RD_NTHANDLE handle;
 	DIR *dirp;
 	int flags, mode;
-	char path[256];
+	char path[PATH_MAX];
 	struct stat filestat;
 
 	handle = 0;
@@ -356,6 +356,19 @@ disk_create(uint32 device_id, uint32 accessmask, uint32 sharemode, uint32 create
 		filename[strlen(filename) - 1] = 0;
 	sprintf(path, "%s%s", g_rdpdr_device[device_id].local_path, filename);
 
+	/* Protect against mailicous servers:
+	   somelongpath/..     not allowed
+	   somelongpath/../b   not allowed
+	   somelongpath/..b    in principle ok, but currently not allowed
+	   somelongpath/b..    ok
+	   somelongpath/b..b   ok
+	   somelongpath/b../c  ok
+	 */
+	if (strstr(path, "/.."))
+	{
+		return RD_STATUS_ACCESS_DENIED;
+	}
+
 	switch (create_disposition)
 	{
 		case CREATE_ALWAYS:
@@ -395,7 +408,7 @@ disk_create(uint32 device_id, uint32 accessmask, uint32 sharemode, uint32 create
 	if ((stat(path, &filestat) == 0) && (S_ISDIR(filestat.st_mode)))
 	{
 		if (flags_and_attributes & FILE_NON_DIRECTORY_FILE)
-			return STATUS_FILE_IS_A_DIRECTORY;
+			return RD_STATUS_FILE_IS_A_DIRECTORY;
 		else
 			flags_and_attributes |= FILE_DIRECTORY_FILE;
 	}
@@ -414,16 +427,16 @@ disk_create(uint32 device_id, uint32 accessmask, uint32 sharemode, uint32 create
 			{
 				case EACCES:
 
-					return STATUS_ACCESS_DENIED;
+					return RD_STATUS_ACCESS_DENIED;
 
 				case ENOENT:
 
-					return STATUS_NO_SUCH_FILE;
+					return RD_STATUS_NO_SUCH_FILE;
 
 				default:
 
 					perror("opendir");
-					return STATUS_NO_SUCH_FILE;
+					return RD_STATUS_NO_SUCH_FILE;
 			}
 		}
 		handle = DIRFD(dirp);
@@ -452,22 +465,22 @@ disk_create(uint32 device_id, uint32 accessmask, uint32 sharemode, uint32 create
 			{
 				case EISDIR:
 
-					return STATUS_FILE_IS_A_DIRECTORY;
+					return RD_STATUS_FILE_IS_A_DIRECTORY;
 
 				case EACCES:
 
-					return STATUS_ACCESS_DENIED;
+					return RD_STATUS_ACCESS_DENIED;
 
 				case ENOENT:
 
-					return STATUS_NO_SUCH_FILE;
+					return RD_STATUS_NO_SUCH_FILE;
 				case EEXIST:
 
-					return STATUS_OBJECT_NAME_COLLISION;
+					return RD_STATUS_OBJECT_NAME_COLLISION;
 				default:
 
 					perror("open");
-					return STATUS_NO_SUCH_FILE;
+					return RD_STATUS_NO_SUCH_FILE;
 			}
 		}
 
@@ -480,7 +493,7 @@ disk_create(uint32 device_id, uint32 accessmask, uint32 sharemode, uint32 create
 	{
 		error("Maximum number of open files (%s) reached. Increase MAX_OPEN_FILES!\n",
 		      handle);
-		exit(1);
+		exit(EX_SOFTWARE);
 	}
 
 	if (dirp)
@@ -491,38 +504,41 @@ disk_create(uint32 device_id, uint32 accessmask, uint32 sharemode, uint32 create
 	g_fileinfo[handle].device_id = device_id;
 	g_fileinfo[handle].flags_and_attributes = flags_and_attributes;
 	g_fileinfo[handle].accessmask = accessmask;
-	strncpy(g_fileinfo[handle].path, path, 255);
+	strncpy(g_fileinfo[handle].path, path, PATH_MAX - 1);
 	g_fileinfo[handle].delete_on_close = False;
-	g_notify_stamp = True;
+
+	if (accessmask & GENERIC_ALL || accessmask & GENERIC_WRITE)
+		g_notify_stamp = True;
 
 	*phandle = handle;
-	return STATUS_SUCCESS;
+	return RD_STATUS_SUCCESS;
 }
 
-static NTSTATUS
-disk_close(NTHANDLE handle)
+static RD_NTSTATUS
+disk_close(RD_NTHANDLE handle)
 {
 	struct fileinfo *pfinfo;
 
 	pfinfo = &(g_fileinfo[handle]);
 
-	g_notify_stamp = True;
+	if (pfinfo->accessmask & GENERIC_ALL || pfinfo->accessmask & GENERIC_WRITE)
+		g_notify_stamp = True;
 
-	rdpdr_abort_io(handle, 0, STATUS_CANCELLED);
+	rdpdr_abort_io(handle, 0, RD_STATUS_CANCELLED);
 
 	if (pfinfo->pdir)
 	{
 		if (closedir(pfinfo->pdir) < 0)
 		{
 			perror("closedir");
-			return STATUS_INVALID_HANDLE;
+			return RD_STATUS_INVALID_HANDLE;
 		}
 
 		if (pfinfo->delete_on_close)
 			if (rmdir(pfinfo->path) < 0)
 			{
 				perror(pfinfo->path);
-				return STATUS_ACCESS_DENIED;
+				return RD_STATUS_ACCESS_DENIED;
 			}
 		pfinfo->delete_on_close = False;
 	}
@@ -531,23 +547,23 @@ disk_close(NTHANDLE handle)
 		if (close(handle) < 0)
 		{
 			perror("close");
-			return STATUS_INVALID_HANDLE;
+			return RD_STATUS_INVALID_HANDLE;
 		}
 		if (pfinfo->delete_on_close)
 			if (unlink(pfinfo->path) < 0)
 			{
 				perror(pfinfo->path);
-				return STATUS_ACCESS_DENIED;
+				return RD_STATUS_ACCESS_DENIED;
 			}
 
 		pfinfo->delete_on_close = False;
 	}
 
-	return STATUS_SUCCESS;
+	return RD_STATUS_SUCCESS;
 }
 
-static NTSTATUS
-disk_read(NTHANDLE handle, uint8 * data, uint32 length, uint32 offset, uint32 * result)
+static RD_NTSTATUS
+disk_read(RD_NTHANDLE handle, uint8 * data, uint32 length, uint32 offset, uint32 * result)
 {
 	int n;
 
@@ -574,20 +590,20 @@ disk_read(NTHANDLE handle, uint8 * data, uint32 length, uint32 offset, uint32 *
 				/* Implement 24 Byte directory read ??
 				   with STATUS_NOT_IMPLEMENTED server doesn't read again */
 				/* return STATUS_FILE_IS_A_DIRECTORY; */
-				return STATUS_NOT_IMPLEMENTED;
+				return RD_STATUS_NOT_IMPLEMENTED;
 			default:
 				perror("read");
-				return STATUS_INVALID_PARAMETER;
+				return RD_STATUS_INVALID_PARAMETER;
 		}
 	}
 
 	*result = n;
 
-	return STATUS_SUCCESS;
+	return RD_STATUS_SUCCESS;
 }
 
-static NTSTATUS
-disk_write(NTHANDLE handle, uint8 * data, uint32 length, uint32 offset, uint32 * result)
+static RD_NTSTATUS
+disk_write(RD_NTHANDLE handle, uint8 * data, uint32 length, uint32 offset, uint32 * result)
 {
 	int n;
 
@@ -602,19 +618,19 @@ disk_write(NTHANDLE handle, uint8 * data, uint32 length, uint32 offset, uint32 *
 		switch (errno)
 		{
 			case ENOSPC:
-				return STATUS_DISK_FULL;
+				return RD_STATUS_DISK_FULL;
 			default:
-				return STATUS_ACCESS_DENIED;
+				return RD_STATUS_ACCESS_DENIED;
 		}
 	}
 
 	*result = n;
 
-	return STATUS_SUCCESS;
+	return RD_STATUS_SUCCESS;
 }
 
-NTSTATUS
-disk_query_information(NTHANDLE handle, uint32 info_class, STREAM out)
+RD_NTSTATUS
+disk_query_information(RD_NTHANDLE handle, uint32 info_class, STREAM out)
 {
 	uint32 file_attributes, ft_high, ft_low;
 	struct stat filestat;
@@ -627,7 +643,7 @@ disk_query_information(NTHANDLE handle, uint32 info_class, STREAM out)
 	{
 		perror("stat");
 		out_uint8(out, 0);
-		return STATUS_ACCESS_DENIED;
+		return RD_STATUS_ACCESS_DENIED;
 	}
 
 	/* Set file attributes */
@@ -689,16 +705,16 @@ disk_query_information(NTHANDLE handle, uint32 info_class, STREAM out)
 		default:
 
 			unimpl("IRP Query (File) Information class: 0x%x\n", info_class);
-			return STATUS_INVALID_PARAMETER;
+			return RD_STATUS_INVALID_PARAMETER;
 	}
-	return STATUS_SUCCESS;
+	return RD_STATUS_SUCCESS;
 }
 
-NTSTATUS
-disk_set_information(NTHANDLE handle, uint32 info_class, STREAM in, STREAM out)
+RD_NTSTATUS
+disk_set_information(RD_NTHANDLE handle, uint32 info_class, STREAM in, STREAM out)
 {
-	uint32 length, file_attributes, ft_high, ft_low, delete_on_close;
-	char newname[256], fullpath[256];
+	uint32 length, file_attributes, ft_high, ft_low;
+	char newname[PATH_MAX], fullpath[PATH_MAX];
 	struct fileinfo *pfinfo;
 	int mode;
 	struct stat filestat;
@@ -742,7 +758,7 @@ disk_set_information(NTHANDLE handle, uint32 info_class, STREAM in, STREAM out)
 			in_uint32_le(in, file_attributes);
 
 			if (fstat(handle, &filestat))
-				return STATUS_ACCESS_DENIED;
+				return RD_STATUS_ACCESS_DENIED;
 
 			tvs.modtime = filestat.st_mtime;
 			tvs.actime = filestat.st_atime;
@@ -768,7 +784,7 @@ disk_set_information(NTHANDLE handle, uint32 info_class, STREAM in, STREAM out)
 				       ctime(&tvs.modtime));
 #endif
 				if (utime(pfinfo->path, &tvs) && errno != EPERM)
-					return STATUS_ACCESS_DENIED;
+					return RD_STATUS_ACCESS_DENIED;
 			}
 
 			if (!file_attributes)
@@ -787,7 +803,7 @@ disk_set_information(NTHANDLE handle, uint32 info_class, STREAM in, STREAM out)
 #endif
 
 			if (fchmod(handle, mode))
-				return STATUS_ACCESS_DENIED;
+				return RD_STATUS_ACCESS_DENIED;
 
 			break;
 
@@ -799,12 +815,12 @@ disk_set_information(NTHANDLE handle, uint32 info_class, STREAM in, STREAM out)
 
 			if (length && (length / 2) < 256)
 			{
-				rdp_in_unistr(in, newname, length);
+				rdp_in_unistr(in, newname, sizeof(newname), length);
 				convert_to_unix_filename(newname);
 			}
 			else
 			{
-				return STATUS_INVALID_PARAMETER;
+				return RD_STATUS_INVALID_PARAMETER;
 			}
 
 			sprintf(fullpath, "%s%s", g_rdpdr_device[pfinfo->device_id].local_path,
@@ -813,7 +829,7 @@ disk_set_information(NTHANDLE handle, uint32 info_class, STREAM in, STREAM out)
 			if (rename(pfinfo->path, fullpath) != 0)
 			{
 				perror("rename");
-				return STATUS_ACCESS_DENIED;
+				return RD_STATUS_ACCESS_DENIED;
 			}
 			break;
 
@@ -827,12 +843,45 @@ disk_set_information(NTHANDLE handle, uint32 info_class, STREAM in, STREAM out)
 			   the delete. See
 			   http://www.osronline.com/article.cfm?article=245. */
 
-			in_uint32_le(in, delete_on_close);
+			/* FileDispositionInformation always sets delete_on_close to true.
+			   "STREAM in" includes Length(4bytes) , Padding(24bytes) and SetBuffer(zero byte).
+			   Length is always set to zero.
+			   [MS-RDPEFS] http://msdn.microsoft.com/en-us/library/cc241305%28PROT.10%29.aspx
+			   - 2.2.3.3.9 Server Drive Set Information Request
+			 */
+			in_uint8s(in, 4);	/* length of SetBuffer */
+			in_uint8s(in, 24);	/* padding */
 
-			if (delete_on_close ||
-			    (pfinfo->
-			     accessmask & (FILE_DELETE_ON_CLOSE | FILE_COMPLETE_IF_OPLOCKED)))
+
+			if ((pfinfo->accessmask &
+			     (FILE_DELETE_ON_CLOSE | FILE_COMPLETE_IF_OPLOCKED)))
 			{
+				/* if file exists in directory , necessary to return RD_STATUS_DIRECTORY_NOT_EMPTY with win2008
+				   [MS-RDPEFS] http://msdn.microsoft.com/en-us/library/cc241305%28PROT.10%29.aspx
+				   - 2.2.3.3.9 Server Drive Set Information Request
+				   - 2.2.3.4.9 Client Drive Set Information Response
+				   [MS-FSCC] http://msdn.microsoft.com/en-us/library/cc231987%28PROT.10%29.aspx
+				   - 2.4.11 FileDispositionInformation
+				   [FSBO] http://msdn.microsoft.com/en-us/library/cc246487%28PROT.13%29.aspx
+				   - 4.3.2 Set Delete-on-close using FileDispositionInformation Information Class (IRP_MJ_SET_INFORMATION)
+				 */
+				if (pfinfo->pdir)
+				{
+					DIR *dp = opendir(pfinfo->path);
+					struct dirent *dir;
+
+					while ((dir = readdir(dp)) != NULL)
+					{
+						if (strcmp(dir->d_name, ".") != 0
+						    && strcmp(dir->d_name, "..") != 0)
+						{
+							closedir(dp);
+							return RD_STATUS_DIRECTORY_NOT_EMPTY;
+						}
+					}
+					closedir(dp);
+				}
+
 				pfinfo->delete_on_close = True;
 			}
 
@@ -850,48 +899,48 @@ disk_set_information(NTHANDLE handle, uint32 info_class, STREAM in, STREAM out)
 			in_uint32_le(in, length);	/* file size */
 
 			/* prevents start of writing if not enough space left on device */
-			if (STATFS_FN(g_rdpdr_device[pfinfo->device_id].local_path, &stat_fs) == 0)
+			if (STATFS_FN(pfinfo->path, &stat_fs) == 0)
 				if (stat_fs.f_bfree * stat_fs.f_bsize < length)
-					return STATUS_DISK_FULL;
+					return RD_STATUS_DISK_FULL;
 
 			if (ftruncate_growable(handle, length) != 0)
 			{
-				return STATUS_DISK_FULL;
+				return RD_STATUS_DISK_FULL;
 			}
 
 			break;
 		default:
 
 			unimpl("IRP Set File Information class: 0x%x\n", info_class);
-			return STATUS_INVALID_PARAMETER;
+			return RD_STATUS_INVALID_PARAMETER;
 	}
-	return STATUS_SUCCESS;
+	return RD_STATUS_SUCCESS;
 }
 
-NTSTATUS
-disk_check_notify(NTHANDLE handle)
+RD_NTSTATUS
+disk_check_notify(RD_NTHANDLE handle)
 {
 	struct fileinfo *pfinfo;
-	NTSTATUS status = STATUS_PENDING;
+	RD_NTSTATUS status = RD_STATUS_PENDING;
 
 	NOTIFY notify;
 
 	pfinfo = &(g_fileinfo[handle]);
 	if (!pfinfo->pdir)
-		return STATUS_INVALID_DEVICE_REQUEST;
+		return RD_STATUS_INVALID_DEVICE_REQUEST;
 
 
 
 	status = NotifyInfo(handle, pfinfo->info_class, &notify);
 
-	if (status != STATUS_PENDING)
+	if (status != RD_STATUS_PENDING)
 		return status;
 
 	if (memcmp(&pfinfo->notify, &notify, sizeof(NOTIFY)))
 	{
 		/*printf("disk_check_notify found changed event\n"); */
 		memcpy(&pfinfo->notify, &notify, sizeof(NOTIFY));
-		status = STATUS_NOTIFY_ENUM_DIR;
+		status = RD_STATUS_NOTIFY_ENUM_DIR;
 	}
 
 	return status;
@@ -899,12 +948,12 @@ disk_check_notify(NTHANDLE handle)
 
 }
 
-NTSTATUS
-disk_create_notify(NTHANDLE handle, uint32 info_class)
+RD_NTSTATUS
+disk_create_notify(RD_NTHANDLE handle, uint32 info_class)
 {
 
 	struct fileinfo *pfinfo;
-	NTSTATUS ret = STATUS_PENDING;
+	RD_NTSTATUS ret = RD_STATUS_PENDING;
 
 	/* printf("start disk_create_notify info_class %X\n", info_class); */
 
@@ -915,8 +964,8 @@ disk_create_notify(NTHANDLE handle, uint32 info_class)
 
 	if (info_class & 0x1000)
 	{			/* ???? */
-		if (ret == STATUS_PENDING)
-			return STATUS_SUCCESS;
+		if (ret == RD_STATUS_PENDING)
+			return RD_STATUS_SUCCESS;
 	}
 
 	/* printf("disk_create_notify: num_entries %d\n", pfinfo->notify.num_entries); */
@@ -926,23 +975,23 @@ disk_create_notify(NTHANDLE handle, uint32 info_class)
 
 }
 
-static NTSTATUS
-NotifyInfo(NTHANDLE handle, uint32 info_class, NOTIFY * p)
+static RD_NTSTATUS
+NotifyInfo(RD_NTHANDLE handle, uint32 info_class, NOTIFY * p)
 {
 	struct fileinfo *pfinfo;
-	struct stat buf;
+	struct stat filestat;
 	struct dirent *dp;
 	char *fullname;
 	DIR *dpr;
 
 	pfinfo = &(g_fileinfo[handle]);
-	if (fstat(handle, &buf) < 0)
+	if (fstat(handle, &filestat) < 0)
 	{
 		perror("NotifyInfo");
-		return STATUS_ACCESS_DENIED;
+		return RD_STATUS_ACCESS_DENIED;
 	}
-	p->modify_time = buf.st_mtime;
-	p->status_time = buf.st_ctime;
+	p->modify_time = filestat.st_mtime;
+	p->status_time = filestat.st_ctime;
 	p->num_entries = 0;
 	p->total_time = 0;
 
@@ -951,7 +1000,7 @@ NotifyInfo(NTHANDLE handle, uint32 info_class, NOTIFY * p)
 	if (!dpr)
 	{
 		perror("NotifyInfo");
-		return STATUS_ACCESS_DENIED;
+		return RD_STATUS_ACCESS_DENIED;
 	}
 
 
@@ -960,19 +1009,19 @@ NotifyInfo(NTHANDLE handle, uint32 info_class, NOTIFY * p)
 		if (!strcmp(dp->d_name, ".") || !strcmp(dp->d_name, ".."))
 			continue;
 		p->num_entries++;
-		fullname = xmalloc(strlen(pfinfo->path) + strlen(dp->d_name) + 2);
+		fullname = (char *) xmalloc(strlen(pfinfo->path) + strlen(dp->d_name) + 2);
 		sprintf(fullname, "%s/%s", pfinfo->path, dp->d_name);
 
-		if (!stat(fullname, &buf))
+		if (!stat(fullname, &filestat))
 		{
-			p->total_time += (buf.st_mtime + buf.st_ctime);
+			p->total_time += (filestat.st_mtime + filestat.st_ctime);
 		}
 
 		xfree(fullname);
 	}
 	closedir(dpr);
 
-	return STATUS_PENDING;
+	return RD_STATUS_PENDING;
 }
 
 static FsInfoType *
@@ -997,7 +1046,7 @@ FsVolumeInfo(char *fpath)
 
 	while ((e = getmntent(fdfs)))
 	{
-		if (strncmp(fpath, e->mnt_dir, strlen(fpath)) == 0)
+		if (str_startswith(e->mnt_dir, fpath))
 		{
 			strcpy(info.type, e->mnt_type);
 			strcpy(info.name, e->mnt_fsname);
@@ -1016,13 +1065,13 @@ FsVolumeInfo(char *fpath)
 						info.serial =
 							(buf[42] << 24) + (buf[41] << 16) +
 							(buf[40] << 8) + buf[39];
-						strncpy(info.label, buf + 43, 10);
+						strncpy(info.label, (char *) buf + 43, 10);
 						info.label[10] = '\0';
 					}
 					else if (lseek(fd, 32767, SEEK_SET) >= 0)	/* ISO9660 */
 					{
 						read(fd, buf, sizeof(buf));
-						strncpy(info.label, buf + 41, 32);
+						strncpy(info.label, (char *) buf + 41, 32);
 						info.label[32] = '\0';
 						/* info.Serial = (buf[128]<<24)+(buf[127]<<16)+(buf[126]<<8)+buf[125]; */
 					}
@@ -1043,8 +1092,8 @@ FsVolumeInfo(char *fpath)
 }
 
 
-NTSTATUS
-disk_query_volume_information(NTHANDLE handle, uint32 info_class, STREAM out)
+RD_NTSTATUS
+disk_query_volume_information(RD_NTHANDLE handle, uint32 info_class, STREAM out)
 {
 	struct STATFS_T stat_fs;
 	struct fileinfo *pfinfo;
@@ -1055,7 +1104,7 @@ disk_query_volume_information(NTHANDLE handle, uint32 info_class, STREAM out)
 	if (STATFS_FN(pfinfo->path, &stat_fs) != 0)
 	{
 		perror("statfs");
-		return STATUS_ACCESS_DENIED;
+		return RD_STATUS_ACCESS_DENIED;
 	}
 
 	fsinfo = FsVolumeInfo(pfinfo->path);
@@ -1084,6 +1133,18 @@ disk_query_volume_information(NTHANDLE handle, uint32 info_class, STREAM out)
 			out_uint32_le(out, 0x200);	/* Bytes per sector */
 			break;
 
+		case FileFsFullSizeInformation:
+
+			out_uint32_le(out, stat_fs.f_blocks);	/* Total allocation units low */
+			out_uint32_le(out, 0);	/* Total allocation units high */
+			out_uint32_le(out, stat_fs.f_blocks);	/* Caller allocation units low */
+			out_uint32_le(out, 0);	/* Caller allocation units high */
+			out_uint32_le(out, stat_fs.f_bfree);	/* Available allocation units */
+			out_uint32_le(out, 0);	/* Available allowcation units */
+			out_uint32_le(out, stat_fs.f_bsize / 0x200);	/* Sectors per allocation unit */
+			out_uint32_le(out, 0x200);	/* Bytes per sector */
+			break;
+
 		case FileFsAttributeInformation:
 
 			out_uint32_le(out, FS_CASE_SENSITIVE | FS_CASE_IS_PRESERVED);	/* fs attributes */
@@ -1096,26 +1157,25 @@ disk_query_volume_information(NTHANDLE handle, uint32 info_class, STREAM out)
 		case FileFsLabelInformation:
 		case FileFsDeviceInformation:
 		case FileFsControlInformation:
-		case FileFsFullSizeInformation:
 		case FileFsObjectIdInformation:
 		case FileFsMaximumInformation:
 
 		default:
 
 			unimpl("IRP Query Volume Information class: 0x%x\n", info_class);
-			return STATUS_INVALID_PARAMETER;
+			return RD_STATUS_INVALID_PARAMETER;
 	}
-	return STATUS_SUCCESS;
+	return RD_STATUS_SUCCESS;
 }
 
-NTSTATUS
-disk_query_directory(NTHANDLE handle, uint32 info_class, char *pattern, STREAM out)
+RD_NTSTATUS
+disk_query_directory(RD_NTHANDLE handle, uint32 info_class, char *pattern, STREAM out)
 {
 	uint32 file_attributes, ft_low, ft_high;
-	char *dirname, fullpath[256];
+	char *dirname, fullpath[PATH_MAX];
 	DIR *pdir;
 	struct dirent *pdirent;
-	struct stat fstat;
+	struct stat filestat;
 	struct fileinfo *pfinfo;
 
 	pfinfo = &(g_fileinfo[handle]);
@@ -1123,14 +1183,18 @@ disk_query_directory(NTHANDLE handle, uint32 info_class, char *pattern, STREAM o
 	dirname = pfinfo->path;
 	file_attributes = 0;
 
+
 	switch (info_class)
 	{
 		case FileBothDirectoryInformation:
+		case FileDirectoryInformation:
+		case FileFullDirectoryInformation:
+		case FileNamesInformation:
 
 			/* If a search pattern is received, remember this pattern, and restart search */
 			if (pattern[0] != 0)
 			{
-				strncpy(pfinfo->pattern, 1 + strrchr(pattern, '/'), 64);
+				strncpy(pfinfo->pattern, 1 + strrchr(pattern, '/'), PATH_MAX - 1);
 				rewinddir(pdir);
 			}
 
@@ -1140,12 +1204,12 @@ disk_query_directory(NTHANDLE handle, uint32 info_class, char *pattern, STREAM o
 				pdirent = readdir(pdir);
 
 			if (pdirent == NULL)
-				return STATUS_NO_MORE_FILES;
+				return RD_STATUS_NO_MORE_FILES;
 
 			/* Get information for directory entry */
 			sprintf(fullpath, "%s/%s", dirname, pdirent->d_name);
 
-			if (stat(fullpath, &fstat))
+			if (stat(fullpath, &filestat))
 			{
 				switch (errno)
 				{
@@ -1153,76 +1217,155 @@ disk_query_directory(NTHANDLE handle, uint32 info_class, char *pattern, STREAM o
 					case ELOOP:
 					case EACCES:
 						/* These are non-fatal errors. */
-						memset(&fstat, 0, sizeof(fstat));
+						memset(&filestat, 0, sizeof(filestat));
 						break;
 					default:
 						/* Fatal error. By returning STATUS_NO_SUCH_FILE, 
 						   the directory list operation will be aborted */
 						perror(fullpath);
 						out_uint8(out, 0);
-						return STATUS_NO_SUCH_FILE;
+						return RD_STATUS_NO_SUCH_FILE;
 				}
 			}
 
-			if (S_ISDIR(fstat.st_mode))
+			if (S_ISDIR(filestat.st_mode))
 				file_attributes |= FILE_ATTRIBUTE_DIRECTORY;
 			if (pdirent->d_name[0] == '.')
 				file_attributes |= FILE_ATTRIBUTE_HIDDEN;
 			if (!file_attributes)
 				file_attributes |= FILE_ATTRIBUTE_NORMAL;
-			if (!(fstat.st_mode & S_IWUSR))
+			if (!(filestat.st_mode & S_IWUSR))
 				file_attributes |= FILE_ATTRIBUTE_READONLY;
 
 			/* Return requested information */
-			out_uint8s(out, 8);	/* unknown zero */
-
-			seconds_since_1970_to_filetime(get_create_time(&fstat), &ft_high, &ft_low);
-			out_uint32_le(out, ft_low);	/* create time */
-			out_uint32_le(out, ft_high);
-
-			seconds_since_1970_to_filetime(fstat.st_atime, &ft_high, &ft_low);
-			out_uint32_le(out, ft_low);	/* last_access_time */
-			out_uint32_le(out, ft_high);
-
-			seconds_since_1970_to_filetime(fstat.st_mtime, &ft_high, &ft_low);
-			out_uint32_le(out, ft_low);	/* last_write_time */
-			out_uint32_le(out, ft_high);
-
-			seconds_since_1970_to_filetime(fstat.st_ctime, &ft_high, &ft_low);
-			out_uint32_le(out, ft_low);	/* change_write_time */
-			out_uint32_le(out, ft_high);
-
-			out_uint32_le(out, fstat.st_size);	/* filesize low */
-			out_uint32_le(out, 0);	/* filesize high */
-			out_uint32_le(out, fstat.st_size);	/* filesize low */
-			out_uint32_le(out, 0);	/* filesize high */
-			out_uint32_le(out, file_attributes);
-			out_uint8(out, 2 * strlen(pdirent->d_name) + 2);	/* unicode length */
-			out_uint8s(out, 7);	/* pad? */
-			out_uint8(out, 0);	/* 8.3 file length */
-			out_uint8s(out, 2 * 12);	/* 8.3 unicode length */
-			rdp_out_unistr(out, pdirent->d_name, 2 * strlen(pdirent->d_name));
+			out_uint32_le(out, 0);	/* NextEntryOffset */
+			out_uint32_le(out, 0);	/* FileIndex zero */
 			break;
 
 		default:
-			/* FIXME: Support FileDirectoryInformation,
-			   FileFullDirectoryInformation, and
-			   FileNamesInformation */
-
 			unimpl("IRP Query Directory sub: 0x%x\n", info_class);
-			return STATUS_INVALID_PARAMETER;
+			return RD_STATUS_INVALID_PARAMETER;
 	}
 
-	return STATUS_SUCCESS;
+	switch (info_class)
+	{
+		case FileBothDirectoryInformation:
+
+			seconds_since_1970_to_filetime(get_create_time(&filestat), &ft_high,
+						       &ft_low);
+			out_uint32_le(out, ft_low);	/* create time */
+			out_uint32_le(out, ft_high);
+
+			seconds_since_1970_to_filetime(filestat.st_atime, &ft_high, &ft_low);
+			out_uint32_le(out, ft_low);	/* last_access_time */
+			out_uint32_le(out, ft_high);
+
+			seconds_since_1970_to_filetime(filestat.st_mtime, &ft_high, &ft_low);
+			out_uint32_le(out, ft_low);	/* last_write_time */
+			out_uint32_le(out, ft_high);
+
+			seconds_since_1970_to_filetime(filestat.st_ctime, &ft_high, &ft_low);
+			out_uint32_le(out, ft_low);	/* change_write_time */
+			out_uint32_le(out, ft_high);
+
+			out_uint32_le(out, filestat.st_size);	/* filesize low */
+			out_uint32_le(out, 0);	/* filesize high */
+			out_uint32_le(out, filestat.st_size);	/* filesize low */
+			out_uint32_le(out, 0);	/* filesize high */
+			out_uint32_le(out, file_attributes);	/* FileAttributes */
+			out_uint32_le(out, 2 * strlen(pdirent->d_name) + 2);	/* unicode length */
+			out_uint32_le(out, 0);	/* EaSize */
+			out_uint8(out, 0);	/* ShortNameLength */
+			/* this should be correct according to MS-FSCC specification
+			   but it only works when commented out... */
+			/* out_uint8(out, 0); *//* Reserved/Padding */
+			out_uint8s(out, 2 * 12);	/* ShortName (8.3 name) */
+			rdp_out_unistr(out, pdirent->d_name, 2 * strlen(pdirent->d_name));
+			break;
+
+
+		case FileDirectoryInformation:
+
+			seconds_since_1970_to_filetime(get_create_time(&filestat), &ft_high,
+						       &ft_low);
+			out_uint32_le(out, ft_low);	/* create time */
+			out_uint32_le(out, ft_high);
+
+			seconds_since_1970_to_filetime(filestat.st_atime, &ft_high, &ft_low);
+			out_uint32_le(out, ft_low);	/* last_access_time */
+			out_uint32_le(out, ft_high);
+
+			seconds_since_1970_to_filetime(filestat.st_mtime, &ft_high, &ft_low);
+			out_uint32_le(out, ft_low);	/* last_write_time */
+			out_uint32_le(out, ft_high);
+
+			seconds_since_1970_to_filetime(filestat.st_ctime, &ft_high, &ft_low);
+			out_uint32_le(out, ft_low);	/* change_write_time */
+			out_uint32_le(out, ft_high);
+
+			out_uint32_le(out, filestat.st_size);	/* filesize low */
+			out_uint32_le(out, 0);	/* filesize high */
+			out_uint32_le(out, filestat.st_size);	/* filesize low */
+			out_uint32_le(out, 0);	/* filesize high */
+			out_uint32_le(out, file_attributes);
+			out_uint32_le(out, 2 * strlen(pdirent->d_name) + 2);	/* unicode length */
+			rdp_out_unistr(out, pdirent->d_name, 2 * strlen(pdirent->d_name));
+			break;
+
+
+		case FileFullDirectoryInformation:
+
+			seconds_since_1970_to_filetime(get_create_time(&filestat), &ft_high,
+						       &ft_low);
+			out_uint32_le(out, ft_low);	/* create time */
+			out_uint32_le(out, ft_high);
+
+			seconds_since_1970_to_filetime(filestat.st_atime, &ft_high, &ft_low);
+			out_uint32_le(out, ft_low);	/* last_access_time */
+			out_uint32_le(out, ft_high);
+
+			seconds_since_1970_to_filetime(filestat.st_mtime, &ft_high, &ft_low);
+			out_uint32_le(out, ft_low);	/* last_write_time */
+			out_uint32_le(out, ft_high);
+
+			seconds_since_1970_to_filetime(filestat.st_ctime, &ft_high, &ft_low);
+			out_uint32_le(out, ft_low);	/* change_write_time */
+			out_uint32_le(out, ft_high);
+
+			out_uint32_le(out, filestat.st_size);	/* filesize low */
+			out_uint32_le(out, 0);	/* filesize high */
+			out_uint32_le(out, filestat.st_size);	/* filesize low */
+			out_uint32_le(out, 0);	/* filesize high */
+			out_uint32_le(out, file_attributes);
+			out_uint32_le(out, 2 * strlen(pdirent->d_name) + 2);	/* unicode length */
+			out_uint32_le(out, 0);	/* EaSize */
+			rdp_out_unistr(out, pdirent->d_name, 2 * strlen(pdirent->d_name));
+			break;
+
+
+		case FileNamesInformation:
+
+			out_uint32_le(out, 2 * strlen(pdirent->d_name) + 2);	/* unicode length */
+			rdp_out_unistr(out, pdirent->d_name, 2 * strlen(pdirent->d_name));
+			break;
+
+
+		default:
+
+			unimpl("IRP Query Directory sub: 0x%x\n", info_class);
+			return RD_STATUS_INVALID_PARAMETER;
+	}
+
+	return RD_STATUS_SUCCESS;
 }
 
 
 
-static NTSTATUS
-disk_device_control(NTHANDLE handle, uint32 request, STREAM in, STREAM out)
+static RD_NTSTATUS
+disk_device_control(RD_NTHANDLE handle, uint32 request, STREAM in, STREAM out)
 {
 	if (((request >> 16) != 20) || ((request >> 16) != 9))
-		return STATUS_INVALID_PARAMETER;
+		return RD_STATUS_INVALID_PARAMETER;
 
 	/* extract operation */
 	request >>= 2;
@@ -1236,10 +1379,10 @@ disk_device_control(NTHANDLE handle, uint32 request, STREAM in, STREAM out)
 		case 42:	/* ? */
 		default:
 			unimpl("DISK IOCTL %d\n", request);
-			return STATUS_INVALID_PARAMETER;
+			return RD_STATUS_INVALID_PARAMETER;
 	}
 
-	return STATUS_SUCCESS;
+	return RD_STATUS_SUCCESS;
 }
 
 DEVICE_FNS disk_fns = {
