@@ -1,21 +1,21 @@
 /* -*- c-basic-offset: 8 -*-
    rdesktop: A Remote Desktop Protocol client.
    Protocol services - Multipoint Communications Service
-   Copyright (C) Matthew Chapman 1999-2005
-   
-   This program is free software; you can redistribute it and/or modify
+   Copyright (C) Matthew Chapman <matthewc.unsw.edu.au> 1999-2008
+   Copyright 2005-2011 Peter Astrand <astrand@cendio.se> for Cendio AB
+
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "rdesktop.h"
@@ -25,7 +25,7 @@ extern VCHANNEL g_channels[];
 extern unsigned int g_num_channels;
 
 /* Parse an ASN.1 BER header */
-static BOOL
+static RD_BOOL
 ber_parse_header(STREAM s, int tagval, int *length)
 {
 	int tag, len;
@@ -36,7 +36,8 @@ ber_parse_header(STREAM s, int tagval, int *length)
 	}
 	else
 	{
-	in_uint8(s, tag)}
+		in_uint8(s, tag);
+	}
 
 	if (tag != tagval)
 	{
@@ -105,7 +106,7 @@ mcs_out_domain_params(STREAM s, int max_channels, int max_users, int max_tokens,
 }
 
 /* Parse a DOMAIN_PARAMS structure (ASN.1 BER) */
-static BOOL
+static RD_BOOL
 mcs_parse_domain_params(STREAM s)
 {
 	int length;
@@ -147,7 +148,7 @@ mcs_send_connect_initial(STREAM mcs_data)
 }
 
 /* Expect a MCS_CONNECT_RESPONSE message (ASN.1 BER) */
-static BOOL
+static RD_BOOL
 mcs_recv_connect_response(STREAM mcs_data)
 {
 	uint8 result;
@@ -221,7 +222,7 @@ mcs_send_aurq(void)
 }
 
 /* Expect a AUcf message (ASN.1 PER) */
-static BOOL
+static RD_BOOL
 mcs_recv_aucf(uint16 * mcs_userid)
 {
 	uint8 opcode, result;
@@ -270,7 +271,7 @@ mcs_send_cjrq(uint16 chanid)
 }
 
 /* Expect a CJcf message (ASN.1 PER) */
-static BOOL
+static RD_BOOL
 mcs_recv_cjcf(void)
 {
 	uint8 opcode, result;
@@ -371,13 +372,12 @@ mcs_recv(uint16 * channel, uint8 * rdpver)
 	return s;
 }
 
-/* Establish a connection up to the MCS layer */
-BOOL
-mcs_connect(char *server, STREAM mcs_data, char *username)
+RD_BOOL
+mcs_connect(char *server, STREAM mcs_data, char *username, RD_BOOL reconnect)
 {
 	unsigned int i;
 
-	if (!iso_connect(server, username))
+	if (!iso_connect(server, username, reconnect))
 		return False;
 
 	mcs_send_connect_initial(mcs_data);
@@ -418,3 +418,11 @@ mcs_disconnect(void)
 {
 	iso_disconnect();
 }
+
+/* reset the state of the mcs layer */
+void
+mcs_reset_state(void)
+{
+	g_mcs_userid = 0;
+	iso_reset_state();
+}
