@@ -1,11 +1,11 @@
-/*
+/* -*- c-basic-offset: 8 -*-
    rdesktop: A Remote Desktop Protocol client.
    Persistent Bitmap Cache routines
-   Copyright (C) Jeroen Meijer 2004-2005
+   Copyright (C) Jeroen Meijer <jeroen@oldambt7.com> 2004-2008
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "rdesktop.h"
@@ -24,14 +23,14 @@
 
 #define IS_PERSISTENT(id) (id < 8 && g_pstcache_fd[id] > 0)
 
-extern int g_server_bpp;
-extern BOOL g_bitmap_cache;
-extern BOOL g_bitmap_cache_persist_enable;
-extern BOOL g_bitmap_cache_precache;
+extern int g_server_depth;
+extern RD_BOOL g_bitmap_cache;
+extern RD_BOOL g_bitmap_cache_persist_enable;
+extern RD_BOOL g_bitmap_cache_precache;
 
 int g_pstcache_fd[8];
 int g_pstcache_Bpp;
-BOOL g_pstcache_enumerated = False;
+RD_BOOL g_pstcache_enumerated = False;
 uint8 zero_key[] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
 
@@ -50,13 +49,13 @@ pstcache_touch_bitmap(uint8 cache_id, uint16 cache_idx, uint32 stamp)
 }
 
 /* Load a bitmap from the persistent cache */
-BOOL
+RD_BOOL
 pstcache_load_bitmap(uint8 cache_id, uint16 cache_idx)
 {
 	uint8 *celldata;
 	int fd;
 	CELLHEADER cellhdr;
-	HBITMAP bitmap;
+	RD_HBITMAP bitmap;
 
 	if (!g_bitmap_cache_persist_enable)
 		return False;
@@ -71,7 +70,7 @@ pstcache_load_bitmap(uint8 cache_id, uint16 cache_idx)
 	rd_read_file(fd, celldata, cellhdr.length);
 
 	bitmap = ui_create_bitmap(cellhdr.width, cellhdr.height, celldata);
-	DEBUG(("Load bitmap from disk: id=%d, idx=%d, bmp=0x%x)\n", cache_id, cache_idx, bitmap));
+	DEBUG(("Load bitmap from disk: id=%d, idx=%d, bmp=%p)\n", cache_id, cache_idx, bitmap));
 	cache_put_bitmap(cache_id, cache_idx, bitmap);
 
 	xfree(celldata);
@@ -79,9 +78,9 @@ pstcache_load_bitmap(uint8 cache_id, uint16 cache_idx)
 }
 
 /* Store a bitmap in the persistent cache */
-BOOL
+RD_BOOL
 pstcache_save_bitmap(uint8 cache_id, uint16 cache_idx, uint8 * key,
-		     uint16 width, uint16 height, uint16 length, uint8 * data)
+		     uint8 width, uint8 height, uint16 length, uint8 * data)
 {
 	int fd;
 	CELLHEADER cellhdr;
@@ -107,7 +106,8 @@ pstcache_save_bitmap(uint8 cache_id, uint16 cache_idx, uint8 * key,
 int
 pstcache_enumerate(uint8 id, HASH_KEY * keylist)
 {
-	int fd, idx, n;
+	int fd, n;
+	uint16 idx;
 	sint16 mru_idx[0xa00];
 	uint32 mru_stamp[0xa00];
 	CELLHEADER cellhdr;
@@ -131,8 +131,8 @@ pstcache_enumerate(uint8 id, HASH_KEY * keylist)
 		{
 			memcpy(keylist[idx], cellhdr.key, sizeof(HASH_KEY));
 
-			/* Pre-cache (not possible for 8bpp because 8bpp needs a colourmap) */
-			if (g_bitmap_cache_precache && cellhdr.stamp && g_server_bpp > 8)
+			/* Pre-cache (not possible for 8 bit colour depth cause it needs a colourmap) */
+			if (g_bitmap_cache_precache && cellhdr.stamp && g_server_depth > 8)
 				pstcache_load_bitmap(id, idx);
 
 			/* Sort by stamp */
@@ -159,7 +159,7 @@ pstcache_enumerate(uint8 id, HASH_KEY * keylist)
 }
 
 /* initialise the persistent bitmap cache */
-BOOL
+RD_BOOL
 pstcache_init(uint8 cache_id)
 {
 	int fd;
@@ -179,7 +179,7 @@ pstcache_init(uint8 cache_id)
 		return False;
 	}
 
-	g_pstcache_Bpp = (g_server_bpp + 7) / 8;
+	g_pstcache_Bpp = (g_server_depth + 7) / 8;
 	sprintf(filename, "cache/pstcache_%d_%d", cache_id, g_pstcache_Bpp);
 	DEBUG(("persistent bitmap cache file: %s\n", filename));
 
