@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 8 -*-
    rdesktop: A Remote Desktop Protocol client.
    Protocol services - Clipboard functions
-   Copyright (C) Erik Forsberg <forsberg@cendio.se> 2003-2008
+   Copyright 2003-2008 Erik Forsberg <forsberg@cendio.se> for Cendio AB
    Copyright (C) Matthew Chapman <matthewc.unsw.edu.au> 2003-2008
    Copyright 2006-2011 Pierre Ossman <ossman@cendio.se> for Cendio AB
 
@@ -389,7 +389,7 @@ xclip_send_data_with_convert(uint8 * source, size_t source_size, Atom target)
 		translated_data = lf2crlf(source, &length);
 		if (translated_data != NULL)
 		{
-			helper_cliprdr_send_response(translated_data, length);
+			helper_cliprdr_send_response(translated_data, length+1);
 			xfree(translated_data);	/* Not the same thing as XFree! */
 		}
 
