@@ -9,3 +9,8 @@ int ewmh_get_window_desktop(Window wnd);
 void ewmh_set_wm_name(Window wnd, const char *title);
 int ewmh_set_window_popup(Window wnd);
 int ewmh_set_window_modal(Window wnd);
+void ewmh_set_icon(Window wnd, int width, int height, const char *rgba_data);
+void ewmh_del_icon(Window wnd, int width, int height);
+int ewmh_set_window_above(Window wnd);
+void set_keypress_keysym(unsigned int keycode, KeySym keysym);
+KeySym reset_keypress_keysym(unsigned int keycode, KeySym keysym);
